
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
library work;
use work.pack_mips.all;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY tbw IS
END tbw;

ARCHITECTURE testbench_arch OF tbw IS
    COMPONENT banc
        PORT (
            clock : In std_logic;
            reset : In std_logic;
            reg_src1 : In std_logic_vector (4 DownTo 0);
            reg_src2 : In std_logic_vector (4 DownTo 0);
            reg_dest : In std_logic_vector (4 DownTo 0);
            donnee : In std_logic_vector (31 DownTo 0);
            cmd_ecr : In std_logic;
            data_src1 : Out std_logic_vector (31 DownTo 0);
            data_src2 : Out std_logic_vector (31 DownTo 0);
            stop_all : Out std_logic
        );
    END COMPONENT;

    SIGNAL clock : std_logic := '0';
    SIGNAL reset : std_logic := '1';
    SIGNAL reg_src1 : std_logic_vector (4 DownTo 0) := "00000";
    SIGNAL reg_src2 : std_logic_vector (4 DownTo 0) := "00000";
    SIGNAL reg_dest : std_logic_vector (4 DownTo 0) := "00000";
    SIGNAL donnee : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL cmd_ecr : std_logic := '0';
    SIGNAL data_src1 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL data_src2 : std_logic_vector (31 DownTo 0) := "00000000000000000000000000000000";
    SIGNAL stop_all : std_logic := '';

    SHARED VARIABLE TX_ERROR : INTEGER := 0;
    SHARED VARIABLE TX_OUT : LINE;

    constant PERIOD : time := 100 ns;
    constant DUTY_CYCLE : real := 0.5;
    constant OFFSET : time := 0 ns;

    BEGIN
        UUT : banc
        PORT MAP (
            clock => clock,
            reset => reset,
            reg_src1 => reg_src1,
            reg_src2 => reg_src2,
            reg_dest => reg_dest,
            donnee => donnee,
            cmd_ecr => cmd_ecr,
            data_src1 => data_src1,
            data_src2 => data_src2,
            stop_all => stop_all
        );

        PROCESS    -- clock process for clock
        BEGIN
            WAIT for OFFSET;
            CLOCK_LOOP : LOOP
                clock <= '0';
                WAIT FOR (PERIOD - (PERIOD * DUTY_CYCLE));
                clock <= '1';
                WAIT FOR (PERIOD * DUTY_CYCLE);
            END LOOP CLOCK_LOOP;
        END PROCESS;

        PROCESS
            PROCEDURE CHECK_data_src1(
                next_data_src1 : std_logic_vector (31 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (data_src1 /= next_data_src1) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns data_src1="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, data_src1);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_data_src1);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_data_src2(
                next_data_src2 : std_logic_vector (31 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (data_src2 /= next_data_src2) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns data_src2="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, data_src2);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_data_src2);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_stop_all(
                next_stop_all : std_logic;
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (stop_all /= next_stop_all) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns stop_all="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, stop_all);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_stop_all);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            BEGIN
                -- -------------  Current Time:  49ns
                WAIT FOR 49 ns;
                reg_dest <= "11111";
                -- -------------------------------------
                -- -------------  Current Time:  149ns
                WAIT FOR 100 ns;
                reset <= '0';
                reg_src2 <= "00010";
                -- -------------------------------------
                -- -------------  Current Time:  249ns
                WAIT FOR 100 ns;
                cmd_ecr <= '1';
                donnee <= "00000000000000000000000000001000";
                -- -------------------------------------
                -- -------------  Current Time:  349ns
                WAIT FOR 100 ns;
                cmd_ecr <= '0';
                reg_src1 <= "11111";
                -- -------------------------------------
                -- -------------  Current Time:  360ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000000111", 360);
                -- -------------------------------------
                -- -------------  Current Time:  460ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000110", 460);
                -- -------------------------------------
                -- -------------  Current Time:  560ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000101", 560);
                -- -------------------------------------
                -- -------------  Current Time:  660ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000100", 660);
                -- -------------------------------------
                -- -------------  Current Time:  760ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000011", 760);
                -- -------------------------------------
                -- -------------  Current Time:  849ns
                WAIT FOR 89 ns;
                cmd_ecr <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  860ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000000010", 860);
                -- -------------------------------------
                -- -------------  Current Time:  960ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000001", 960);
                -- -------------------------------------
                -- -------------  Current Time:  1049ns
                WAIT FOR 89 ns;
                donnee <= "00000000000000000000000000000100";
                -- -------------------------------------
                -- -------------  Current Time:  1060ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000000000", 1060);
                -- -------------------------------------
                -- -------------  Current Time:  1149ns
                WAIT FOR 89 ns;
                cmd_ecr <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  1160ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000001000", 1160);
                -- -------------------------------------
                -- -------------  Current Time:  1260ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000111", 1260);
                -- -------------------------------------
                -- -------------  Current Time:  1349ns
                WAIT FOR 89 ns;
                cmd_ecr <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  1360ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000000110", 1360);
                -- -------------------------------------
                -- -------------  Current Time:  1460ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000101", 1460);
                -- -------------------------------------
                -- -------------  Current Time:  1560ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000100", 1560);
                -- -------------------------------------
                -- -------------  Current Time:  1660ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000011", 1660);
                -- -------------------------------------
                -- -------------  Current Time:  1760ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000010", 1760);
                -- -------------------------------------
                -- -------------  Current Time:  1860ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000001", 1860);
                -- -------------------------------------
                -- -------------  Current Time:  1960ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000000", 1960);
                -- -------------------------------------
                -- -------------  Current Time:  2249ns
                WAIT FOR 289 ns;
                donnee <= "00000000000000000000000000000101";
                -- -------------------------------------
                -- -------------  Current Time:  3349ns
                WAIT FOR 1100 ns;
                cmd_ecr <= '1';
                donnee <= "00000000000000000000000000001000";
                -- -------------------------------------
                -- -------------  Current Time:  3360ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000001000", 3360);
                -- -------------------------------------
                -- -------------  Current Time:  3449ns
                WAIT FOR 89 ns;
                cmd_ecr <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  3460ns
                WAIT FOR 11 ns;
                CHECK_data_src1("00000000000000000000000000000111", 3460);
                -- -------------------------------------
                -- -------------  Current Time:  3560ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000110", 3560);
                -- -------------------------------------
                -- -------------  Current Time:  3660ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000101", 3660);
                -- -------------------------------------
                -- -------------  Current Time:  3760ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000100", 3760);
                -- -------------------------------------
                -- -------------  Current Time:  3860ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000011", 3860);
                -- -------------------------------------
                -- -------------  Current Time:  3960ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000010", 3960);
                -- -------------------------------------
                -- -------------  Current Time:  4060ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000001", 4060);
                -- -------------------------------------
                -- -------------  Current Time:  4160ns
                WAIT FOR 100 ns;
                CHECK_data_src1("00000000000000000000000000000000", 4160);
                -- -------------------------------------
                WAIT FOR 940 ns;

                IF (TX_ERROR = 0) THEN
                    STD.TEXTIO.write(TX_OUT, string'("No errors or warnings"));
                    ASSERT (FALSE) REPORT
                      "Simulation successful (not a failure).  No problems detected."
                      SEVERITY FAILURE;
                ELSE
                    STD.TEXTIO.write(TX_OUT, TX_ERROR);
                    STD.TEXTIO.write(TX_OUT,
                        string'(" errors found in simulation"));
                    ASSERT (FALSE) REPORT "Errors found during simulation"
                         SEVERITY FAILURE;
                END IF;
            END PROCESS;

    END testbench_arch;

